from typing import Dict, List, Any, Optional, Tuple
from collections import defaultdict, Counter
import json
import numpy as np
from datetime import datetime, timedelta
import asyncio
from sqlalchemy.orm import Session
from app.db.models import UserInteraction, UserPreference, LearningPattern, User
from app.core.config import settings
import logging

logger = logging.getLogger(__name__)


class LearningEngine:
    def __init__(self):
        self.user_patterns = defaultdict(dict)
        self.feedback_weights = {
            -1: -0.2,  # Negative feedback
            0: 0.0,    # Neutral feedback
            1: 0.1     # Positive feedback
        }
        
    async def analyze_user_interactions(self, db: Session, user_id: int) -> Dict[str, Any]:
        """Analyze user interaction patterns to extract learning insights"""
        try:
            # Get recent interactions (last 30 days)
            cutoff_date = datetime.utcnow() - timedelta(days=30)
            interactions = db.query(UserInteraction).filter(
                UserInteraction.user_id == user_id,
                UserInteraction.created_at >= cutoff_date
            ).all()
            
            if not interactions:
                return {}
            
            analysis = {
                'interaction_frequency': self._analyze_frequency_patterns(interactions),
                'topic_preferences': self._analyze_topic_preferences(interactions),
                'response_quality': self._analyze_response_quality(interactions),
                'search_patterns': self._analyze_search_patterns(interactions),
                'time_patterns': self._analyze_time_patterns(interactions)
            }
            
            return analysis
            
        except Exception as e:
            logger.error(f"Failed to analyze user interactions: {e}")
            return {}
    
    def _analyze_frequency_patterns(self, interactions: List[UserInteraction]) -> Dict[str, Any]:
        """Analyze how frequently user interacts with the system"""
        if not interactions:
            return {}
        
        # Group interactions by day
        daily_counts = defaultdict(int)
        for interaction in interactions:
            day = interaction.created_at.date()
            daily_counts[day] += 1
        
        # Calculate patterns
        counts = list(daily_counts.values())
        return {
            'avg_daily_interactions': np.mean(counts) if counts else 0,
            'max_daily_interactions': max(counts) if counts else 0,
            'active_days': len(daily_counts),
            'total_interactions': len(interactions)
        }
    
    def _analyze_topic_preferences(self, interactions: List[UserInteraction]) -> Dict[str, Any]:
        """Analyze what topics user asks about most frequently"""
        topic_keywords = []
        successful_topics = []
        
        for interaction in interactions:
            if interaction.query:
                # Extract keywords from query (simple approach)
                words = interaction.query.lower().split()
                # Filter out common stop words
                stop_words = {'the', 'a', 'an', 'and', 'or', 'but', 'in', 'on', 'at', 'to', 'for', 'of', 'with', 'by', 'is', 'are', 'was', 'were', 'what', 'how', 'when', 'where', 'why', 'who'}
                keywords = [word for word in words if len(word) > 3 and word not in stop_words]
                topic_keywords.extend(keywords)
                
                # Track successful topics (positive feedback)
                if interaction.user_feedback and interaction.user_feedback > 0:
                    successful_topics.extend(keywords)
        
        # Count frequencies
        topic_counts = Counter(topic_keywords)
        successful_counts = Counter(successful_topics)
        
        return {
            'most_common_topics': dict(topic_counts.most_common(10)),
            'successful_topics': dict(successful_counts.most_common(5)),
            'topic_diversity': len(set(topic_keywords))
        }
    
    def _analyze_response_quality(self, interactions: List[UserInteraction]) -> Dict[str, Any]:
        """Analyze response quality based on user feedback"""
        feedback_scores = []
        response_times = []
        helpful_count = 0
        total_feedback = 0
        
        for interaction in interactions:
            if interaction.user_feedback is not None:
                feedback_scores.append(interaction.user_feedback)
                total_feedback += 1
            
            if interaction.was_helpful is not None:
                if interaction.was_helpful:
                    helpful_count += 1
            
            if interaction.response_time:
                response_times.append(interaction.response_time)
        
        return {
            'avg_feedback_score': np.mean(feedback_scores) if feedback_scores else 0,
            'feedback_distribution': dict(Counter(feedback_scores)),
            'helpfulness_rate': helpful_count / total_feedback if total_feedback > 0 else 0,
            'avg_response_time': np.mean(response_times) if response_times else 0,
            'total_feedback_count': total_feedback
        }
    
    def _analyze_search_patterns(self, interactions: List[UserInteraction]) -> Dict[str, Any]:
        """Analyze search and file usage patterns"""
        search_terms = []
        used_files = []
        
        for interaction in interactions:
            if interaction.search_terms:
                search_terms.extend(interaction.search_terms)
            if interaction.used_files:
                used_files.extend(interaction.used_files)
        
        return {
            'common_search_terms': dict(Counter(search_terms).most_common(10)),
            'frequently_used_files': dict(Counter(used_files).most_common(10)),
            'search_diversity': len(set(search_terms))
        }
    
    def _analyze_time_patterns(self, interactions: List[UserInteraction]) -> Dict[str, Any]:
        """Analyze when user is most active"""
        hours = []
        days_of_week = []
        
        for interaction in interactions:
            hours.append(interaction.created_at.hour)
            days_of_week.append(interaction.created_at.weekday())
        
        return {
            'peak_hours': dict(Counter(hours).most_common(5)),
            'active_days_of_week': dict(Counter(days_of_week).most_common()),
            'activity_distribution': {
                'morning': sum(1 for h in hours if 6 <= h < 12),
                'afternoon': sum(1 for h in hours if 12 <= h < 18),
                'evening': sum(1 for h in hours if 18 <= h < 24),
                'night': sum(1 for h in hours if 0 <= h < 6)
            }
        }
    
    async def update_user_preferences(self, db: Session, user_id: int, analysis: Dict[str, Any]):
        """Update user preferences based on learning analysis"""
        try:
            # Update or create preferences based on analysis
            preferences_to_update = [
                ('response_style', 'preferred_length', self._infer_response_length_preference(analysis)),
                ('topics', 'interests', analysis.get('topic_preferences', {}).get('most_common_topics', {})),
                ('interaction', 'peak_hours', analysis.get('time_patterns', {}).get('peak_hours', {})),
                ('quality', 'feedback_history', analysis.get('response_quality', {}))
            ]
            
            for pref_type, pref_key, pref_value in preferences_to_update:
                if pref_value:
                    # Check if preference exists
                    existing_pref = db.query(UserPreference).filter(
                        UserPreference.user_id == user_id,
                        UserPreference.preference_type == pref_type,
                        UserPreference.preference_key == pref_key
                    ).first()
                    
                    confidence_score = self._calculate_confidence_score(analysis, pref_type)
                    
                    if existing_pref:
                        # Update existing preference
                        existing_pref.preference_value = pref_value
                        existing_pref.confidence_score = confidence_score
                        existing_pref.updated_at = datetime.utcnow()
                    else:
                        # Create new preference
                        new_pref = UserPreference(
                            user_id=user_id,
                            preference_type=pref_type,
                            preference_key=pref_key,
                            preference_value=pref_value,
                            confidence_score=confidence_score
                        )
                        db.add(new_pref)
            
            db.commit()
            logger.info(f"Updated preferences for user {user_id}")
            
        except Exception as e:
            logger.error(f"Failed to update user preferences: {e}")
            db.rollback()
    
    def _infer_response_length_preference(self, analysis: Dict[str, Any]) -> str:
        """Infer user's preferred response length based on interaction patterns"""
        response_quality = analysis.get('response_quality', {})
        avg_feedback = response_quality.get('avg_feedback_score', 0)
        
        # Simple heuristic: if user gives positive feedback, maintain current style
        if avg_feedback > 0.5:
            return 'detailed'
        elif avg_feedback < -0.2:
            return 'concise'
        else:
            return 'balanced'
    
    def _calculate_confidence_score(self, analysis: Dict[str, Any], preference_type: str) -> float:
        """Calculate confidence score for a preference based on data volume and consistency"""
        base_confidence = 0.5
        
        # Factor in number of interactions
        total_interactions = analysis.get('interaction_frequency', {}).get('total_interactions', 0)
        interaction_factor = min(total_interactions / 100, 1.0) * 0.3
        
        # Factor in feedback consistency
        response_quality = analysis.get('response_quality', {})
        feedback_count = response_quality.get('total_feedback_count', 0)
        feedback_factor = min(feedback_count / 20, 1.0) * 0.2
        
        return min(base_confidence + interaction_factor + feedback_factor, 1.0)
    
    async def generate_personalized_prompt(self, db: Session, user_id: int, base_prompt: str) -> str:
        """Generate personalized system prompt based on user preferences"""
        try:
            # Get user preferences
            preferences = db.query(UserPreference).filter(
                UserPreference.user_id == user_id
            ).all()
            
            # Build personalization context
            personalization = []
            
            for pref in preferences:
                if pref.confidence_score > 0.6:  # Only use high-confidence preferences
                    if pref.preference_type == 'response_style':
                        if pref.preference_key == 'preferred_length':
                            if pref.preference_value == 'concise':
                                personalization.append("Provide concise, direct answers.")
                            elif pref.preference_value == 'detailed':
                                personalization.append("Provide detailed, comprehensive explanations.")
                    
                    elif pref.preference_type == 'topics':
                        topics = list(pref.preference_value.keys())[:3]  # Top 3 topics
                        if topics:
                            personalization.append(f"User frequently asks about: {', '.join(topics)}")
            
            # Combine base prompt with personalization
            if personalization:
                personal_context = "\n".join(personalization)
                return f"{base_prompt}\n\nPersonalization context:\n{personal_context}"
            
            return base_prompt
            
        except Exception as e:
            logger.error(f"Failed to generate personalized prompt: {e}")
            return base_prompt
    
    async def suggest_proactive_actions(self, db: Session, user_id: int) -> List[Dict[str, Any]]:
        """Suggest proactive actions based on user patterns"""
        try:
            analysis = await self.analyze_user_interactions(db, user_id)
            suggestions = []
            
            # Suggest reminders based on time patterns
            time_patterns = analysis.get('time_patterns', {})
            peak_hours = time_patterns.get('peak_hours', {})
            if peak_hours:
                most_active_hour = max(peak_hours.keys())
                suggestions.append({
                    'type': 'reminder_optimization',
                    'message': f"You're most active at {most_active_hour}:00. Would you like to schedule important reminders around this time?",
                    'confidence': 0.7
                })
            
            # Suggest file organization based on usage patterns
            search_patterns = analysis.get('search_patterns', {})
            frequent_files = search_patterns.get('frequently_used_files', {})
            if frequent_files:
                suggestions.append({
                    'type': 'file_organization',
                    'message': "I noticed you frequently access certain files. Would you like me to create quick access shortcuts?",
                    'confidence': 0.6
                })
            
            # Suggest topic exploration based on interests
            topic_prefs = analysis.get('topic_preferences', {})
            common_topics = topic_prefs.get('most_common_topics', {})
            if common_topics:
                top_topic = max(common_topics.keys(), key=common_topics.get)
                suggestions.append({
                    'type': 'content_discovery',
                    'message': f"You seem interested in {top_topic}. Would you like me to search for related documents in your files?",
                    'confidence': 0.5
                })
            
            return suggestions
            
        except Exception as e:
            logger.error(f"Failed to generate proactive suggestions: {e}")
            return []
    
    async def record_learning_pattern(self, db: Session, user_id: int, pattern_type: str, pattern_data: Dict[str, Any]):
        """Record a new learning pattern for future reference"""
        try:
            pattern = LearningPattern(
                user_id=user_id,
                pattern_type=pattern_type,
                pattern_data=pattern_data,
                confidence_score=0.5,
                usage_count=1,
                success_rate=0.0
            )
            
            db.add(pattern)
            db.commit()
            logger.info(f"Recorded learning pattern for user {user_id}: {pattern_type}")
            
        except Exception as e:
            logger.error(f"Failed to record learning pattern: {e}")
            db.rollback()
    
    async def update_pattern_success(self, db: Session, pattern_id: int, was_successful: bool):
        """Update the success rate of a learning pattern"""
        try:
            pattern = db.query(LearningPattern).filter(LearningPattern.id == pattern_id).first()
            if pattern:
                pattern.usage_count += 1
                current_success_rate = pattern.success_rate * (pattern.usage_count - 1)
                if was_successful:
                    current_success_rate += 1
                pattern.success_rate = current_success_rate / pattern.usage_count
                pattern.updated_at = datetime.utcnow()
                
                db.commit()
                logger.info(f"Updated pattern {pattern_id} success rate: {pattern.success_rate}")
                
        except Exception as e:
            logger.error(f"Failed to update pattern success: {e}")
            db.rollback()


# Global instance
learning_engine = LearningEngine() 