import os
import magic
from pathlib import Path
from typing import Dict, List, Any, Optional, Tuple
import logging
import hashlib
from datetime import datetime

# File processing imports
import PyPDF2
from docx import Document
from PIL import Image
import pytesseract

logger = logging.getLogger(__name__)


class FileProcessor:
    def __init__(self):
        self.supported_types = {
            'application/pdf': self._process_pdf,
            'application/vnd.openxmlformats-officedocument.wordprocessingml.document': self._process_docx,
            'text/plain': self._process_text,
            'text/markdown': self._process_text,
            'image/jpeg': self._process_image,
            'image/png': self._process_image,
            'image/gif': self._process_image,
            'image/bmp': self._process_image,
            'image/tiff': self._process_image,
        }
        
        # Categories for auto-classification
        self.categories = {
            'work': ['project', 'meeting', 'report', 'presentation', 'proposal', 'contract', 'invoice'],
            'personal': ['diary', 'journal', 'note', 'reminder', 'todo', 'list'],
            'financial': ['budget', 'expense', 'income', 'tax', 'receipt', 'bank', 'investment'],
            'education': ['course', 'study', 'lecture', 'assignment', 'exam', 'research'],
            'health': ['medical', 'doctor', 'prescription', 'health', 'fitness', 'exercise'],
            'travel': ['itinerary', 'booking', 'ticket', 'hotel', 'flight', 'vacation'],
            'legal': ['contract', 'agreement', 'legal', 'law', 'court', 'document'],
            'technical': ['code', 'programming', 'software', 'api', 'documentation', 'manual']
        }
    
    async def process_file(self, file_path: str, original_name: str) -> Dict[str, Any]:
        """Process a file and extract relevant information"""
        try:
            # Detect file MIME type
            mime_type = magic.from_file(file_path, mime=True)
            file_size = os.path.getsize(file_path)
            
            # Extract content based on file type
            content_info = {
                'original_name': original_name,
                'file_path': file_path,
                'mime_type': mime_type,
                'file_size': file_size,
                'processed_at': datetime.utcnow().isoformat(),
                'extracted_text': '',
                'content_summary': '',
                'categories': [],
                'metadata': {}
            }
            
            if mime_type in self.supported_types:
                processor = self.supported_types[mime_type]
                extracted_data = await processor(file_path)
                content_info.update(extracted_data)
            else:
                logger.warning(f"Unsupported file type: {mime_type}")
                content_info['error'] = f"Unsupported file type: {mime_type}"
            
            # Auto-categorize content
            if content_info['extracted_text']:
                content_info['categories'] = self._categorize_content(content_info['extracted_text'])
                content_info['content_summary'] = self._generate_summary(content_info['extracted_text'])
            
            return content_info
            
        except Exception as e:
            logger.error(f"Failed to process file {file_path}: {e}")
            return {
                'original_name': original_name,
                'file_path': file_path,
                'error': str(e),
                'processed_at': datetime.utcnow().isoformat()
            }
    
    async def _process_pdf(self, file_path: str) -> Dict[str, Any]:
        """Extract text from PDF files"""
        try:
            extracted_text = ""
            metadata = {}
            
            with open(file_path, 'rb') as file:
                pdf_reader = PyPDF2.PdfReader(file)
                
                # Extract metadata
                if pdf_reader.metadata:
                    metadata = {
                        'title': pdf_reader.metadata.get('/Title', ''),
                        'author': pdf_reader.metadata.get('/Author', ''),
                        'subject': pdf_reader.metadata.get('/Subject', ''),
                        'creator': pdf_reader.metadata.get('/Creator', ''),
                        'creation_date': str(pdf_reader.metadata.get('/CreationDate', '')),
                    }
                
                metadata['page_count'] = len(pdf_reader.pages)
                
                # Extract text from all pages
                for page_num, page in enumerate(pdf_reader.pages):
                    try:
                        text = page.extract_text()
                        if text:
                            extracted_text += f"\n--- Page {page_num + 1} ---\n{text}\n"
                    except Exception as e:
                        logger.warning(f"Failed to extract text from page {page_num + 1}: {e}")
                        continue
            
            return {
                'extracted_text': extracted_text.strip(),
                'metadata': metadata,
                'file_type': 'pdf'
            }
            
        except Exception as e:
            logger.error(f"Failed to process PDF {file_path}: {e}")
            return {'extracted_text': '', 'error': str(e), 'file_type': 'pdf'}
    
    async def _process_docx(self, file_path: str) -> Dict[str, Any]:
        """Extract text from DOCX files"""
        try:
            extracted_text = ""
            metadata = {}
            
            doc = Document(file_path)
            
            # Extract metadata
            core_props = doc.core_properties
            metadata = {
                'title': core_props.title or '',
                'author': core_props.author or '',
                'subject': core_props.subject or '',
                'created': str(core_props.created) if core_props.created else '',
                'modified': str(core_props.modified) if core_props.modified else '',
                'keywords': core_props.keywords or '',
            }
            
            # Extract text from paragraphs
            paragraphs = []
            for paragraph in doc.paragraphs:
                if paragraph.text.strip():
                    paragraphs.append(paragraph.text.strip())
            
            extracted_text = '\n'.join(paragraphs)
            metadata['paragraph_count'] = len(paragraphs)
            
            return {
                'extracted_text': extracted_text,
                'metadata': metadata,
                'file_type': 'docx'
            }
            
        except Exception as e:
            logger.error(f"Failed to process DOCX {file_path}: {e}")
            return {'extracted_text': '', 'error': str(e), 'file_type': 'docx'}
    
    async def _process_text(self, file_path: str) -> Dict[str, Any]:
        """Process plain text files"""
        try:
            with open(file_path, 'r', encoding='utf-8', errors='ignore') as file:
                content = file.read()
            
            # Basic text file metadata
            lines = content.split('\n')
            words = content.split()
            
            metadata = {
                'line_count': len(lines),
                'word_count': len(words),
                'character_count': len(content)
            }
            
            return {
                'extracted_text': content,
                'metadata': metadata,
                'file_type': 'text'
            }
            
        except Exception as e:
            logger.error(f"Failed to process text file {file_path}: {e}")
            return {'extracted_text': '', 'error': str(e), 'file_type': 'text'}
    
    async def _process_image(self, file_path: str) -> Dict[str, Any]:
        """Extract text from images using OCR"""
        try:
            extracted_text = ""
            metadata = {}
            
            # Open image and extract metadata
            with Image.open(file_path) as img:
                metadata = {
                    'format': img.format,
                    'mode': img.mode,
                    'size': img.size,
                    'width': img.width,
                    'height': img.height
                }
                
                # Extract EXIF data if available
                if hasattr(img, '_getexif') and img._getexif():
                    exif_data = img._getexif()
                    if exif_data:
                        metadata['exif'] = {str(k): str(v) for k, v in exif_data.items()}
                
                # Perform OCR to extract text
                try:
                    extracted_text = pytesseract.image_to_string(img)
                    if extracted_text.strip():
                        metadata['has_text'] = True
                    else:
                        metadata['has_text'] = False
                except Exception as ocr_error:
                    logger.warning(f"OCR failed for {file_path}: {ocr_error}")
                    metadata['ocr_error'] = str(ocr_error)
            
            return {
                'extracted_text': extracted_text.strip(),
                'metadata': metadata,
                'file_type': 'image'
            }
            
        except Exception as e:
            logger.error(f"Failed to process image {file_path}: {e}")
            return {'extracted_text': '', 'error': str(e), 'file_type': 'image'}
    
    def _categorize_content(self, text: str) -> List[str]:
        """Automatically categorize content based on keywords"""
        text_lower = text.lower()
        detected_categories = []
        
        for category, keywords in self.categories.items():
            # Count how many keywords from this category appear in the text
            keyword_count = sum(1 for keyword in keywords if keyword in text_lower)
            
            # If more than 20% of keywords are found, or at least 2 keywords, add category
            threshold = max(1, len(keywords) * 0.2)
            if keyword_count >= threshold:
                detected_categories.append(category)
        
        # If no categories detected, try to infer from common patterns
        if not detected_categories:
            if any(word in text_lower for word in ['meeting', 'project', 'deadline', 'task']):
                detected_categories.append('work')
            elif any(word in text_lower for word in ['personal', 'diary', 'thought', 'feeling']):
                detected_categories.append('personal')
            else:
                detected_categories.append('general')
        
        return detected_categories
    
    def _generate_summary(self, text: str, max_length: int = 200) -> str:
        """Generate a simple summary of the content"""
        if not text or len(text) <= max_length:
            return text
        
        # Simple extractive summary: take first few sentences
        sentences = text.split('.')
        summary = ""
        
        for sentence in sentences:
            if len(summary + sentence) <= max_length:
                summary += sentence + "."
            else:
                break
        
        return summary.strip() if summary else text[:max_length] + "..."
    
    def chunk_text(self, text: str, chunk_size: int = 1000, overlap: int = 100) -> List[str]:
        """Split text into overlapping chunks for embedding"""
        if not text:
            return []
        
        words = text.split()
        chunks = []
        
        for i in range(0, len(words), chunk_size - overlap):
            chunk_words = words[i:i + chunk_size]
            chunk_text = ' '.join(chunk_words)
            if chunk_text.strip():
                chunks.append(chunk_text.strip())
        
        return chunks
    
    def get_file_hash(self, file_path: str) -> str:
        """Generate hash of file content for deduplication"""
        try:
            hash_sha256 = hashlib.sha256()
            with open(file_path, 'rb') as f:
                for chunk in iter(lambda: f.read(4096), b""):
                    hash_sha256.update(chunk)
            return hash_sha256.hexdigest()
        except Exception as e:
            logger.error(f"Failed to generate hash for {file_path}: {e}")
            return ""
    
    def is_supported_file(self, file_path: str) -> bool:
        """Check if file type is supported"""
        try:
            mime_type = magic.from_file(file_path, mime=True)
            return mime_type in self.supported_types
        except Exception:
            return False


# Global instance
file_processor = FileProcessor() 