import chromadb
from chromadb.config import Settings
from typing import List, Dict, Any, Optional, Tuple
import uuid
from pathlib import Path
from app.core.config import settings
import logging

logger = logging.getLogger(__name__)


class VectorStore:
    def __init__(self, persist_directory: str = None, collection_name: str = None):
        self.persist_directory = persist_directory or str(settings.VECTOR_DB_DIR)
        self.collection_name = collection_name or settings.VECTOR_COLLECTION_NAME
        self.client = None
        self.collection = None
        self._initialize_client()
    
    def _initialize_client(self):
        """Initialize ChromaDB client and collection"""
        try:
            # Create persistent client
            self.client = chromadb.PersistentClient(
                path=self.persist_directory,
                settings=Settings(anonymized_telemetry=False)
            )
            
            # Get or create collection
            self.collection = self.client.get_or_create_collection(
                name=self.collection_name,
                metadata={"hnsw:space": "cosine"}  # Use cosine similarity
            )
            
            logger.info(f"Vector store initialized with collection: {self.collection_name}")
            
        except Exception as e:
            logger.error(f"Failed to initialize vector store: {e}")
            raise Exception(f"Could not initialize vector database: {e}")
    
    def add_documents(
        self,
        documents: List[str],
        embeddings: List[List[float]],
        metadatas: List[Dict[str, Any]],
        ids: Optional[List[str]] = None
    ) -> List[str]:
        """Add documents with embeddings to the vector store"""
        try:
            if not documents or not embeddings:
                return []
            
            # Generate IDs if not provided
            if ids is None:
                ids = [str(uuid.uuid4()) for _ in documents]
            
            # Ensure all lists have the same length
            if not (len(documents) == len(embeddings) == len(metadatas) == len(ids)):
                raise ValueError("Documents, embeddings, metadatas, and ids must have the same length")
            
            # Add to collection
            self.collection.add(
                documents=documents,
                embeddings=embeddings,
                metadatas=metadatas,
                ids=ids
            )
            
            logger.info(f"Added {len(documents)} documents to vector store")
            return ids
            
        except Exception as e:
            logger.error(f"Failed to add documents to vector store: {e}")
            raise Exception(f"Could not add documents to vector database: {e}")
    
    def search_similar(
        self,
        query_embedding: List[float],
        n_results: int = 5,
        where: Optional[Dict[str, Any]] = None,
        include: List[str] = None
    ) -> Dict[str, List[Any]]:
        """Search for similar documents using embedding"""
        try:
            if include is None:
                include = ["documents", "metadatas", "distances"]
            
            results = self.collection.query(
                query_embeddings=[query_embedding],
                n_results=n_results,
                where=where,
                include=include
            )
            
            # Flatten results since we only query with one embedding
            flattened_results = {}
            for key, values in results.items():
                if values and len(values) > 0:
                    flattened_results[key] = values[0]
                else:
                    flattened_results[key] = []
            
            return flattened_results
            
        except Exception as e:
            logger.error(f"Failed to search vector store: {e}")
            return {"documents": [], "metadatas": [], "distances": []}
    
    def search_by_text(
        self,
        query_text: str,
        n_results: int = 5,
        where: Optional[Dict[str, Any]] = None
    ) -> Dict[str, List[Any]]:
        """Search for similar documents using text query"""
        try:
            results = self.collection.query(
                query_texts=[query_text],
                n_results=n_results,
                where=where,
                include=["documents", "metadatas", "distances"]
            )
            
            # Flatten results
            flattened_results = {}
            for key, values in results.items():
                if values and len(values) > 0:
                    flattened_results[key] = values[0]
                else:
                    flattened_results[key] = []
            
            return flattened_results
            
        except Exception as e:
            logger.error(f"Failed to search vector store by text: {e}")
            return {"documents": [], "metadatas": [], "distances": []}
    
    def get_documents_by_ids(self, ids: List[str]) -> Dict[str, List[Any]]:
        """Retrieve documents by their IDs"""
        try:
            results = self.collection.get(
                ids=ids,
                include=["documents", "metadatas"]
            )
            return results
            
        except Exception as e:
            logger.error(f"Failed to get documents by IDs: {e}")
            return {"documents": [], "metadatas": []}
    
    def update_document(
        self,
        document_id: str,
        document: Optional[str] = None,
        embedding: Optional[List[float]] = None,
        metadata: Optional[Dict[str, Any]] = None
    ) -> bool:
        """Update an existing document"""
        try:
            update_data = {"ids": [document_id]}
            
            if document is not None:
                update_data["documents"] = [document]
            if embedding is not None:
                update_data["embeddings"] = [embedding]
            if metadata is not None:
                update_data["metadatas"] = [metadata]
            
            self.collection.update(**update_data)
            logger.info(f"Updated document: {document_id}")
            return True
            
        except Exception as e:
            logger.error(f"Failed to update document {document_id}: {e}")
            return False
    
    def delete_documents(self, ids: List[str]) -> bool:
        """Delete documents by their IDs"""
        try:
            self.collection.delete(ids=ids)
            logger.info(f"Deleted {len(ids)} documents from vector store")
            return True
            
        except Exception as e:
            logger.error(f"Failed to delete documents: {e}")
            return False
    
    def delete_by_metadata(self, where: Dict[str, Any]) -> bool:
        """Delete documents by metadata criteria"""
        try:
            self.collection.delete(where=where)
            logger.info(f"Deleted documents matching criteria: {where}")
            return True
            
        except Exception as e:
            logger.error(f"Failed to delete documents by metadata: {e}")
            return False
    
    def get_collection_stats(self) -> Dict[str, Any]:
        """Get statistics about the collection"""
        try:
            count = self.collection.count()
            return {
                "collection_name": self.collection_name,
                "total_documents": count,
                "persist_directory": self.persist_directory
            }
            
        except Exception as e:
            logger.error(f"Failed to get collection stats: {e}")
            return {}
    
    def clear_collection(self) -> bool:
        """Clear all documents from the collection"""
        try:
            # Delete the collection and recreate it
            self.client.delete_collection(self.collection_name)
            self.collection = self.client.get_or_create_collection(
                name=self.collection_name,
                metadata={"hnsw:space": "cosine"}
            )
            logger.info(f"Cleared collection: {self.collection_name}")
            return True
            
        except Exception as e:
            logger.error(f"Failed to clear collection: {e}")
            return False
    
    def create_user_collection(self, user_id: int, collection_name: str = None) -> 'VectorStore':
        """Create a user-specific collection"""
        if collection_name is None:
            collection_name = f"user_{user_id}_documents"
        
        return VectorStore(
            persist_directory=self.persist_directory,
            collection_name=collection_name
        )


# Global instance
vector_store = VectorStore() 