import React from 'react'
import { useQuery } from '@tanstack/react-query'
import { 
  MessageSquare, 
  Files, 
  Bell, 
  Activity,
  Upload,
  Search,
  Plus
} from 'lucide-react'
import { Link } from 'react-router-dom'
import { api } from '../services/api'

export default function Dashboard() {
  const { data: usageStats } = useQuery({
    queryKey: ['usage-stats'],
    queryFn: api.getUsageStats,
  })

  const { data: filesData } = useQuery({
    queryKey: ['files', 1, 5],
    queryFn: () => api.getFiles(1, 5),
  })

  const { data: reminders } = useQuery({
    queryKey: ['reminders'],
    queryFn: api.getReminders,
  })

  const { data: suggestions } = useQuery({
    queryKey: ['suggestions'],
    queryFn: api.getProactiveSuggestions,
  })

  const stats = [
    {
      name: 'Total Conversations',
      value: usageStats?.total_conversations || 0,
      icon: MessageSquare,
      color: 'text-blue-500',
    },
    {
      name: 'Files Uploaded',
      value: filesData?.total || 0,
      icon: Files,
      color: 'text-green-500',
    },
    {
      name: 'Active Reminders',
      value: reminders?.filter((r: any) => !r.isCompleted).length || 0,
      icon: Bell,
      color: 'text-yellow-500',
    },
    {
      name: 'Learning Score',
      value: Math.round((usageStats?.learning_score || 0) * 100),
      icon: Activity,
      color: 'text-purple-500',
    },
  ]

  const quickActions = [
    {
      name: 'Start Chat',
      description: 'Ask me anything about your files',
      href: '/chat',
      icon: MessageSquare,
      color: 'bg-blue-500',
    },
    {
      name: 'Upload Files',
      description: 'Add documents for analysis',
      href: '/files',
      icon: Upload,
      color: 'bg-green-500',
    },
    {
      name: 'Search',
      description: 'Find content in your files',
      href: '/files',
      icon: Search,
      color: 'bg-purple-500',
    },
    {
      name: 'Add Reminder',
      description: 'Create a new reminder',
      href: '/reminders',
      icon: Plus,
      color: 'bg-orange-500',
    },
  ]

  return (
    <div className="p-6 space-y-6">
      {/* Header */}
      <div>
        <h1 className="text-2xl font-bold text-foreground">Dashboard</h1>
        <p className="text-muted-foreground">
          Welcome back! Here's what's happening with your AI assistant.
        </p>
      </div>

      {/* Stats Grid */}
      <div className="grid grid-cols-1 md:grid-cols-2 lg:grid-cols-4 gap-6">
        {stats.map((stat) => {
          const Icon = stat.icon
          return (
            <div key={stat.name} className="card p-6">
              <div className="flex items-center justify-between">
                <div>
                  <p className="text-sm font-medium text-muted-foreground">
                    {stat.name}
                  </p>
                  <p className="text-3xl font-bold text-foreground">
                    {stat.value}
                  </p>
                </div>
                <Icon className={`h-8 w-8 ${stat.color}`} />
              </div>
            </div>
          )
        })}
      </div>

      {/* Quick Actions */}
      <div className="card">
        <div className="p-6 border-b">
          <h2 className="text-lg font-semibold text-foreground">Quick Actions</h2>
          <p className="text-sm text-muted-foreground">
            Get started with these common tasks
          </p>
        </div>
        <div className="p-6">
          <div className="grid grid-cols-1 md:grid-cols-2 lg:grid-cols-4 gap-4">
            {quickActions.map((action) => {
              const Icon = action.icon
              return (
                <Link
                  key={action.name}
                  to={action.href}
                  className="flex items-center space-x-3 p-4 rounded-lg border hover:bg-accent transition-colors"
                >
                  <div className={`p-2 rounded-md ${action.color} text-white`}>
                    <Icon className="h-5 w-5" />
                  </div>
                  <div>
                    <p className="font-medium text-foreground">{action.name}</p>
                    <p className="text-xs text-muted-foreground">
                      {action.description}
                    </p>
                  </div>
                </Link>
              )
            })}
          </div>
        </div>
      </div>

      {/* Recent Activity */}
      <div className="grid grid-cols-1 lg:grid-cols-2 gap-6">
        {/* Recent Files */}
        <div className="card">
          <div className="p-6 border-b">
            <h2 className="text-lg font-semibold text-foreground">Recent Files</h2>
          </div>
          <div className="p-6">
            {filesData?.files?.length > 0 ? (
              <div className="space-y-3">
                {filesData.files.slice(0, 5).map((file: any) => (
                  <div key={file.id} className="flex items-center space-x-3">
                    <Files className="h-4 w-4 text-muted-foreground" />
                    <div className="flex-1 min-w-0">
                      <p className="text-sm font-medium text-foreground truncate">
                        {file.originalName}
                      </p>
                      <p className="text-xs text-muted-foreground">
                        {new Date(file.createdAt).toLocaleDateString()}
                      </p>
                    </div>
                  </div>
                ))}
              </div>
            ) : (
              <p className="text-sm text-muted-foreground">
                No files uploaded yet. Start by uploading some documents.
              </p>
            )}
          </div>
        </div>

        {/* AI Suggestions */}
        <div className="card">
          <div className="p-6 border-b">
            <h2 className="text-lg font-semibold text-foreground">AI Suggestions</h2>
          </div>
          <div className="p-6">
            {suggestions?.length > 0 ? (
              <div className="space-y-3">
                {suggestions.slice(0, 3).map((suggestion: any, index: number) => (
                  <div key={index} className="p-3 bg-muted rounded-lg">
                    <p className="text-sm text-foreground">{suggestion.message}</p>
                    <p className="text-xs text-muted-foreground mt-1">
                      Confidence: {Math.round(suggestion.confidence * 100)}%
                    </p>
                  </div>
                ))}
              </div>
            ) : (
              <p className="text-sm text-muted-foreground">
                I'll provide personalized suggestions as you use the system more.
              </p>
            )}
          </div>
        </div>
      </div>
    </div>
  )
} 