import axios from 'axios'

const BASE_URL = import.meta.env.VITE_API_URL || 'http://localhost:8000'

// Create axios instance
const apiClient = axios.create({
  baseURL: BASE_URL,
  timeout: 30000,
})

// Add auth token to requests
apiClient.interceptors.request.use((config) => {
  const token = localStorage.getItem('auth-storage')
  if (token) {
    try {
      const parsed = JSON.parse(token)
      if (parsed.state?.token) {
        config.headers.Authorization = `Bearer ${parsed.state.token}`
      }
    } catch (error) {
      console.error('Failed to parse auth token:', error)
    }
  }
  return config
})

// Handle auth errors
apiClient.interceptors.response.use(
  (response) => response,
  (error) => {
    if (error.response?.status === 401) {
      // Clear auth on 401
      localStorage.removeItem('auth-storage')
      window.location.reload()
    }
    return Promise.reject(error)
  }
)

// Types
export interface SystemInfo {
  app_info: {
    name: string
    version: string
    debug: boolean
  }
  ai_services: {
    embedding_model: any
    vector_store: any
    available_llm_models: string[]
    current_llm_model: string
  }
  capabilities: {
    file_processing: string[]
    ai_features: string[]
  }
}

export interface ChatMessage {
  id: string
  role: 'user' | 'assistant'
  content: string
  timestamp: string
  metadata?: any
}

export interface FileInfo {
  id: number
  filename: string
  originalName: string
  fileType: string
  fileSize: number
  contentSummary?: string
  categories: string[]
  tags: string[]
  createdAt: string
  lastAccessed?: string
}

export interface Reminder {
  id: number
  title: string
  description?: string
  reminderTime: string
  isCompleted: boolean
  isRecurring: boolean
  priority: number
  autoGenerated: boolean
}

export interface SearchResult {
  id: string
  content: string
  metadata: any
  similarity: number
  source: string
}

// API methods
export const api = {
  // System endpoints
  async getHealth() {
    const response = await apiClient.get('/health')
    return response.data
  },

  async getSystemInfo(): Promise<SystemInfo> {
    const response = await apiClient.get('/api/v1/system/info')
    return response.data
  },

  // Auth endpoints
  async login(username: string, password: string) {
    const response = await apiClient.post('/api/v1/auth/login', {
      username,
      password,
    })
    return response.data
  },

  async register(userData: {
    username: string
    email: string
    password: string
    fullName?: string
  }) {
    const response = await apiClient.post('/api/v1/auth/register', userData)
    return response.data
  },

  async logout() {
    await apiClient.post('/api/v1/auth/logout')
  },

  // Chat endpoints
  async sendMessage(message: string): Promise<{ response: string; used_files?: string[] }> {
    const response = await apiClient.post('/api/v1/chat/message', {
      message,
    })
    return response.data
  },

  async getChatHistory(): Promise<ChatMessage[]> {
    const response = await apiClient.get('/api/v1/chat/history')
    return response.data
  },

  async provideFeedback(messageId: string, feedback: number) {
    await apiClient.post(`/api/v1/chat/feedback/${messageId}`, {
      feedback,
    })
  },

  // File endpoints
  async uploadFile(file: File, tags?: string[]) {
    const formData = new FormData()
    formData.append('file', file)
    if (tags) {
      formData.append('tags', JSON.stringify(tags))
    }

    const response = await apiClient.post('/api/v1/files/upload', formData, {
      headers: {
        'Content-Type': 'multipart/form-data',
      },
    })
    return response.data
  },

  async getFiles(page = 1, limit = 20, category?: string): Promise<{ files: FileInfo[]; total: number }> {
    const params = new URLSearchParams({ 
      page: page.toString(), 
      limit: limit.toString() 
    })
    if (category) params.append('category', category)
    
    const response = await apiClient.get(`/api/v1/files?${params}`)
    return response.data
  },

  async deleteFile(fileId: number) {
    await apiClient.delete(`/api/v1/files/${fileId}`)
  },

  async updateFileTags(fileId: number, tags: string[]) {
    const response = await apiClient.patch(`/api/v1/files/${fileId}/tags`, {
      tags,
    })
    return response.data
  },

  // Search endpoints
  async searchFiles(query: string, limit = 10): Promise<SearchResult[]> {
    const response = await apiClient.get('/api/v1/search', {
      params: { query, limit },
    })
    return response.data
  },

  // Reminders endpoints
  async getReminders(): Promise<Reminder[]> {
    const response = await apiClient.get('/api/v1/reminders')
    return response.data
  },

  async createReminder(reminder: Omit<Reminder, 'id' | 'autoGenerated'>) {
    const response = await apiClient.post('/api/v1/reminders', reminder)
    return response.data
  },

  async updateReminder(reminderId: number, updates: Partial<Reminder>) {
    const response = await apiClient.patch(`/api/v1/reminders/${reminderId}`, updates)
    return response.data
  },

  async deleteReminder(reminderId: number) {
    await apiClient.delete(`/api/v1/reminders/${reminderId}`)
  },

  async getProactiveSuggestions() {
    const response = await apiClient.get('/api/v1/suggestions')
    return response.data
  },

  // Analytics endpoints
  async getUsageStats() {
    const response = await apiClient.get('/api/v1/analytics/usage')
    return response.data
  },

  async getInteractionHistory() {
    const response = await apiClient.get('/api/v1/analytics/interactions')
    return response.data
  },
} 